//-------------------------------------------------------------------------------------------------------
//	　　
//     電子ボリューム　ＣＳ３３１０用　２ｃｈ専用　　
//     XC8用に書き換え　2020/12/07
//			 
//	使用PIC　PIC16F1827: 8.0MHz
//  音量＋　　：　音量－　　：　L/R切り替え　：消音
//　
//　	PIC内部での管理番号	メーカー	メーカーコード	UP		down   L/R切り替え  	消音  　
//		0				秋月		10			a0		　00	   	58          d8　L/R切り替えはCボタン
//  	1				NEC　	18			02		　03	   	0a			08
//　 	2				SANYO	30			09		　0a	   	05         	00
//		3				東芝		40			1a		　1e		0f			12
//		4				日立		50			12		　15	   	09			17
//		5				富士通	15			16		　17    	00          12
//		6				パイオニア	AA			0a		　0b		13			1c
//		7				オリオン	80			0c		　0d		12			0f
//		8				LG		04			02	　	　03		0b			08
//		9				d:sign	F1			02		　03		00			08  L/R切り替えは選局upボタン
//														(16進表記)　　　　　　　　　　　　　
//　　　　　　　L/R切り替えは入力切替ボタン、消音は電源ボタンで代用
//-------------------------------------------------------------------------------------------------------
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON      // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)

/* 内部クロック　８ＭＨｚ */
#define  _XTAL_FREQ     8000000		//ディレイ用
	/********　変数定義　************/
static	unsigned int    i;
static	unsigned int	moter; 			// 0:off,　1:ボリュームアップ,2:ボリュームダウン
static	unsigned int	data[36];		//赤外線のHi・low時間の保管用

static	unsigned int	manudata;		//PIC内部でのメーカー番号
const	int	manu[10] = {0x10,0x18,0x30,0x40,0x50,0x15,0xaa,0x80,0x04,0xf1}; 	//メーカーコード
const	int	  up[10] = {0xa0,0x02,0x09,0x1a,0x12,0x16,0x0a,0x0c,0x02,0x02};	//音量UP
const	int	down[10] = {0x00,0x03,0x0a,0x1e,0x15,0x17,0x0b,0x0d,0x03,0x03};	//音量DOWN
const	int	LR_c[10] = {0x58,0x0a,0x05,0x0f,0x09,0x00,0x13,0x12,0x0b,0x00};	//L/R切り替え（入力切替）
const	int	mute_c[10] = {0xd8,0x08,0x00,0x12,0x17,0x12,0x1c,0x0f,0x08,0x08};//消音（電源）

static	unsigned    int	select_mode;    //select 1:Both L/R,2:Only L,3:Only R
static	unsigned    int	mute_mode;		//
static	signed  int     l_dbdata;       //左側デシベル表示用	
static	signed  int     r_dbdata;       //右側デシベル表示用
//static	float	l_dbdata;       //左側デシベル表示用	
//static	float	r_dbdata;       //右側デシベル表示用
static	signed  int		l_vrdata;       //左側音量
static	signed  int		r_vrdata;       //右側音量

static  unsigned    int   r_dec01;
static  signed      int   r_dec10;  
static  unsigned    int   l_dec01;
static  signed      int   l_dec10;

static	unsigned    int     repeat;         //短時間でリピートを繰り返さない用							

#define BOTH	1
#define L_ONLY  2
#define R_ONLY  3

#define MUTE_OFF 0
#define MUTE_ON  1

#define ON	1
#define OFF 0
#define DELAY 97

/********　サブ関数　************/
void    data_out(int out_data);
void	LCD_mode(void);
void	vrup(void);
void	vrdown(void);
void	select(void);
void 	mute(void);
void	manuset(void);
void    putch(unsigned char byte);

/********　メイン開始　************/
void main(void) {
    
    int		inputdata;  	//受信信号からデーターへの変換用
	int		manucord;		//リモコンのメーカー識別用
	int 	Received_data;	//赤外線リモコンのコマンドデータ
	int 	Received_buf;	//1回のコマンドを一時保管、リピート用
	long	starttime = 0;	//起動後の5秒間カウント用
 
    OSCCON = 0x72;          		//内臓クロック８HMz指定(BPLL Off  
    ANSELA = 0x00;          		//A/D停止
    ANSELB = 0x00;              	//A/D停止

    TRISA  = 0b00000001;            //ポートＡ 赤外線入力RA0のみ入力
    TRISB  = 0b00000000;            //ポートＢ すべて出力
 
    /* USART初期設定*/
    //　TXCKSEL　0=RB2　1=RB5　　　RXDTSEL　0=RB1　1=RB2
    TXCKSEL = 1;                    // TX = RB5
    RXDTSEL = 1;                    // Rx = RB2
    TXSTA = 0x24;                   //シリアル受信設定
    RCSTA = 0x90;                   //シリアル送信設定
	BAUDCON=0x08;                   //
	SPBRG = 103;                    //SPBRG=207時9600bps
                                    //SPBRG=103時19200bps

    RB4 = 0;                        //
    RB2 = 0;                        //
    
//　ＣＳ３３１０コントロール 電源投入時　誤動作するので２回データを送信している。
// 一旦ミュートにする。

    RB3 = 0;                // /CS
	data_out(0);
	data_out(0);
    RB3 = 1;                // /CS
    __delay_ms(1);
    RB3 = 0;
	data_out(0);            // /CS
	data_out(0);
    RB3 = 1;                 // /CS
    
//　動作チェック用 ＬＣＤ表示
    __delay_ms(1000);
	printf("L2:  Ver. 1.8      \n");
    __delay_ms(8);
    printf("L1:  CS3310        \n");

//　前回のボリュームデータを内臓ＥＥＰＲＯＭから読み出す。
	l_vrdata = eeprom_read(0x00);	//左側
	r_vrdata = eeprom_read(0x01);	//右側
	if (l_vrdata > 250)				// もしEEPROMのデーターが　250以上だったら
	{								// 1 にする。（書き込みしたばかりの場合　0ｘFFなので）
		l_vrdata = 1;				// 
		r_vrdata = 1;				//
		eeprom_write(0x00,0x01);	//1　をセット
		eeprom_write(0x01,0x01);	//1　をセット
	}
	
	select_mode = 1;						//両方の音量を変更モードにセットする。		

//　cs3310 に前回の音量を送る
    RB3 = 0;
	data_out(r_vrdata);
	data_out(l_vrdata);
    RB3 = 1;
//=============================================================	
//    電源投入後　5秒間に今後使用したいリモコンを操作する（ボタンはどれでもいい）	
//    
// 	　使用するのリモコンのメーカーをセット
//
//=============================================================

	manudata = eeprom_read(0x02);
	//　EEPROMのデータが0xffの場合　秋月電子製をセットする。
	if	(manudata == 0xff)
	{
		eeprom_write(0x02,0x00);
		manudata = 0x00;
	}

//	printf("EEPROM DATA = %x\r\n",manudata);
	
	while(starttime < 5000)
	{
        if(RA0)    
		{
			__delay_ms(1);
			starttime++ ;
		}
		
		else
		{
			manuset();
			printf("L1:   Change!!!    \n");
			__delay_ms(8);
			printf("L2:                \n");
			__delay_ms(1500);
			break; //ループから抜ける
		}
	}
		
//使用するメーカーの確認

	LCD_mode();

//受光モジュールからの信号待ち（スタートビット待ち）
	while(1)
	{
		while(RA0) { __delay_ms(1);}	//スタートビット待ち
		//　配列を初期値に戻す。		
		for (i = 0;i < 35;i++ )
		{
			data[i]=0;
		}		
		inputdata = 0xff;
		moter = 0;

//    	リーダーコード　
//		スタートビットの長さを測る  初回送信　9ms　  		
		while(RA0 == 0) {            
			__delay_us(DELAY);
			data[0]++;
		}	 
	
//		スタートビット後のLOW時間、　初回送信　4.5ms　/　リピート　2.25ms
        while(RA0) {
			__delay_us(DELAY);
			data[1]++;
		}
//		1回目のコマンド　だったら　データを取り込む	
		if (data[0] > 80 && data[1] > 40){
			for(i = 0;i < 26  ;i++)
			{	
				while(RA0 == 0) {}
				while(RA0) {
					__delay_us(DELAY);
					data[2+i]++;
				}				
			}	
//　データコードのみ取り出す。
			
			inputdata = 0;				
			if (data[18]>10){inputdata ++;}
			if (data[19]>10){inputdata = inputdata + 2;}
			if (data[20]>10){inputdata = inputdata + 4;}
			if (data[21]>10){inputdata = inputdata + 8;}
			if (data[22]>10){inputdata = inputdata + 16;}
			if (data[23]>10){inputdata = inputdata + 32;}
			if (data[24]>10){inputdata = inputdata + 64;}
			if (data[25]>10){inputdata = inputdata + 128;}			
	//		printf("L1:inputdata %X \r\n",inputdata);	

//メーカー識別
			manucord = 0;
			if (data[2]>10){manucord ++;}
			if (data[3]>10){manucord = manucord + 2;}
			if (data[4]>10){manucord = manucord + 4;}
			if (data[5]>10){manucord = manucord + 8;}
			if (data[6]>10){manucord = manucord + 16;}
			if (data[7]>10){manucord = manucord + 32;}
			if (data[8]>10){manucord = manucord + 64;}
			if (data[9]>10){manucord = manucord + 128;}
	//		printf("L2:manucord  :%X \r\n",manucord);	
	
			moter = 0;
			Received_buf = 0;
			
//ｃｓ３３１０　操作

			if (manu[manudata] == manucord)		//受信
			{
				repeat = 0 ;

				if(up[manudata] == inputdata)
				{
					Received_buf = 1;

					vrup();
				}					

				if(down[manudata] == inputdata)
				{
					Received_buf = 2;

					vrdown();
				}					
					if(LR_c[manudata] == inputdata)
				{

					Received_buf = 0;
					select();
				}					
				if(mute_c[manudata] == inputdata)
				{

					Received_buf = 0;
					mute();
				}					
			}
		}
				
// リピートデータか？
		else if(data[0] > 80 && data[1] > 20)
		{

				if(Received_buf == 1)
				{
				     vrup();
				}
				else if(Received_buf == 2)
				{
					vrdown();
				}	
		}					
	}	//　マイコン内部ループ
}		//　メイン

/*=======================================================
*   メーカー識別セット
*=======================================================*/
void	manuset(void)
{
	int	manucord;
	
//		スタートビットの長さを測る  初回送信　9ms　  		
		while(RA0 == 0) {
		}	 
	
//		スタートビット後のLOW時間、　初回送信　4.5ms　/　リピート　2.25ms
		while(RA0) { 
			}
//		1回目のコマンド　だったら　データを取り込む	
			for(i = 0;i < 26  ;i++)
			{	
				while(RA0 == 0) {}	 
				
				while(RA0) { 
					__delay_us(DELAY);
					data[2+i]++;
				}				
			}
			
//メーカー判別
			manucord = 0;
			if (data[2]>10){manucord ++;}
			if (data[3]>10){manucord = manucord + 2;}
			if (data[4]>10){manucord = manucord + 4;}
			if (data[5]>10){manucord = manucord + 8;}
			if (data[6]>10){manucord = manucord + 16;}
			if (data[7]>10){manucord = manucord + 32;}
			if (data[8]>10){manucord = manucord + 64;}
			if (data[9]>10){manucord = manucord + 128;}
			
			switch (manucord)
			{
				case 0x10: manudata = 0;break;	//秋月電子
				case 0x18: manudata = 1;break;	//NEC
				case 0x30: manudata = 2;break;	//サンヨー
				case 0x40: manudata = 3;break;	//東芝
				case 0x50: manudata = 4;break;	//日立
				case 0x15: manudata = 5;break;	//富士通				
				case 0xAA: manudata = 6;break;	//パイオニア
				case 0x80: manudata = 7;break;	//オリオン							
				case 0x04: manudata = 8;break;	//LG
				case 0xF1: manudata = 9;break;	//ｂｙ：D:sign　（不明メーカー）
			}
			eeprom_write(0x02,manudata);	
}

/*=======================================================
*	音量　アップ　処理　
*=======================================================*/
void	vrup(void)
{
	mute_mode = 0;

	switch(select_mode)
	{
		case 1://左右
			if (l_vrdata <= 192) { l_vrdata ++; }			
			if (r_vrdata <= 192) { r_vrdata ++; }			
		break;

		case 2://左
			if (l_vrdata <= 192) { l_vrdata ++; }			
		break;

		case 3://右
			if (r_vrdata <= 192) { r_vrdata ++; }	
		break;
	}

	//LCD 表示													
	LCD_mode();
	//eeprom にボリューム値を書き込む
	eeprom_write(0x00,l_vrdata);				
	eeprom_write(0x01,r_vrdata);		
    
	//// CS3310 へデーターを送る
	RB3 = 0;
	data_out(r_vrdata);
	data_out(l_vrdata);
    RB3 = 1;

}
/*=======================================================
*	音量　ダウン　処理　
*=======================================================*/
void	vrdown(void)
{
	mute_mode = 0;

	switch(select_mode)
	{
		case 1://左右
			if (l_vrdata > 1) { l_vrdata --; }			
			if (r_vrdata > 1) { r_vrdata --; }			
		break;

		case 2://左
			if (l_vrdata > 1) { l_vrdata --; }			
		break;

		case 3://右
			if (r_vrdata > 1) { r_vrdata --; }			
		break;	
	}

	//LCD 表示													
	LCD_mode();
	//eeprom にボリューム値を書き込む
	eeprom_write(0x00,l_vrdata);				
	eeprom_write(0x01,r_vrdata);				
	// CS3310 へデーターを送る

    RB3 = 0;
	data_out(r_vrdata);
	data_out(l_vrdata);

    RB3 = 1;

}

/*=======================================================
*　　　　LCD 音量表示　
*=======================================================*/
void	LCD_mode(void)
{
  //  printf("l_dbdata = %X\r\n",l_dbdata);
  //   printf("r_dbdata = %X\r\n",r_dbdata);
    
	l_dbdata = 315 - (0.5 * (2550 - l_vrdata*10));
    l_dec10 = l_dbdata / 10;
    l_dec01 = abs(l_dbdata % 10);    

	r_dbdata = 315 - (0.5 * (2550 - r_vrdata*10));
    r_dec10 = r_dbdata / 10;
    r_dec01 = abs(r_dbdata % 10);
     
	switch(select_mode)
	{
		case 1:
			printf("L1:*LEFT  %d.%d dB \n\r",l_dec10,l_dec01);
            __delay_ms(8);
			printf("L2:*RIGHT %d.%d dB \n\r",r_dec10,r_dec01);
		break;
		case 2:
			printf("L1:*LEFT  %d.%d dB \n\r",l_dec10,l_dec01);
            __delay_ms(8);
			printf("L2: RIGHT %d.%d dB \n\r",r_dec10,r_dec01);
		break;

		case 3:
			printf("L1: LEFT  %d.%d dB \n\r",l_dec10,l_dec01);
            __delay_ms(8);
			printf("L2:*RIGHT %d.%d dB \n\r",r_dec10,r_dec01);
		break;
	}
}
//================================================
// CS3310 へ　データー　送信
//================================================
void	data_out(int out_data)
{
	int sd[9];
	
	sd[0] = (0x01 & out_data);
	sd[0] >>=0;
	sd[1] = (0x02 & out_data);
	sd[1] >>=1;
	sd[2] = (0x04 & out_data);
	sd[2] >>=2;
	sd[3] = (0x08 & out_data);
	sd[3] >>=3;
	sd[4] = (0x10 & out_data);
	sd[4] >>=4;
	sd[5] = (0x20 & out_data);
	sd[5] >>=5;
	sd[6] = (0x40 & out_data);
	sd[6] >>=6;
	sd[7] = (0x80 & out_data);
	sd[7] >>=7;

    /****** bit7 ******/
    RB2 = sd[7];                //sdata
    RB4 = 1;                    //sclk
    
	/****** bit6 ******/
    RB2 = sd[6];
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;
    
	/****** bit5 ******/
    RB2 = sd[5];
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;
    
	/****** bit4 ******/    
    RB2 = sd[4]; 
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;
       
	/****** bit3 ******/    
    RB2 = sd[3];
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;
       
	/****** bit2 ******/    
    RB2 = sd[2];
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;
       
	/****** bit1 ******/    
    RB2 = sd[1];
    RB4 = 0;
     __delay_us(4);
    RB4 = 1;
       
	/****** bit0 ******/    
    RB2 = sd[0];
    RB4 = 0;
    __delay_us(4);
    RB4 = 1;                 
    __delay_us(4);       
    
    RB4 = 0;                 //データ送信が終了したので信号をlowにする。
    RB2 = 0;
}

/*=======================================================
*  左右切り替え
*=======================================================*/
void	select(void)
{
	mute_mode = 0;
	select_mode++;
	if (select_mode >3){select_mode = 1;}
																	
	LCD_mode();         //LCD 表示		
	//// CS3310 へデーターを送る
    RB3 = 0;
	data_out(r_vrdata);
	data_out(l_vrdata);
    RB3 = 1;
}

/*=======================================================
*  消音処理
*=======================================================*/
void 	mute(void)
{
	if (mute_mode == 0)     //消音処理	
	{
		//// CS3310 へデーターを送る
        RB3 = 0;
		data_out(0);
		data_out(0);
        RB3 = 1;
		printf("L1:*  MUTE         \n");
        __delay_ms(8);
		printf("L2:*  MUTE         \n");
		
		mute_mode = 1;
	}
	else				//復帰処理
	{
		l_vrdata = eeprom_read(0x00);	//左側
		r_vrdata = eeprom_read(0x01);	//右側
	
        RB3 = 0;
		data_out(r_vrdata);
		data_out(l_vrdata);
        RB3 = 1;
		LCD_mode();
		mute_mode = 0;
	}
    __delay_ms(100);
}

 /************************************************************
 * シリアル通信　printf用
 ***********************************************************/
void putch(unsigned char byte){
		RCSTAbits.SPEN=1;
		TXREG=byte;
		while(!TXIF)continue;
		TXIF=0;
}